import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { WeeklyAdRepository } from "./weekly-ad.repository";
import { BaseWeeklyAdDto } from "./dto/base-weekly-ad.dto";
import { WeeklyAdEntity } from "./entities/weekly-ad.entity";
import { UpdateWeeklyAdDto } from "./dto/update-weekly-ad.dto";
import { WeeklyAdTagsListRepository } from "../weekly-ad-tags-list/weekly-ad-tags-list.repository";

@Injectable()
export class WeeklyAdService {
    constructor(
      @InjectRepository(WeeklyAdRepository)
      private readonly repository: WeeklyAdRepository,
      private readonly doSpaceService: DoSpacesService,
      @InjectRepository(WeeklyAdTagsListRepository)
      private readonly weeklyAdTagsListRepository: WeeklyAdTagsListRepository,
    ) {
    }

    async create(dto: BaseWeeklyAdDto, files: {
        backgroundImage: UploadedMulterFileI
    }): Promise<WeeklyAdEntity | Error> {
        if (files && files.backgroundImage) {
            const backgroundImage: any = await this.doSpaceService.uploadFile(files.backgroundImage[0], "WeeklyAd/");
            dto.backgroundImage = backgroundImage;
        }
        
        const data: any = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
              Constants.CREATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
      }

      if (data) {
        for (let tag of JSON.parse(dto.tags)) {
          const res = await this.weeklyAdTagsListRepository.createEntity({
            weeklyAdId: data?.id,
            tagName: tag.tagName,
            tagId: tag.tagId
          })

          if (!res) {
            throw new HttpException(
              Constants.CREATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
          }
        }

        const tags = await this.weeklyAdTagsListRepository.findByFilterQuery({ weeklyAdId: data.id });

        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", { ...data, tags });
      }
    }

    async update(
      id: string,
      dto: UpdateWeeklyAdDto,
      files: {
          backgroundImage: UploadedMulterFileI
      }
    ): Promise<WeeklyAdEntity | Error> {
      const res: any = await this.repository.findOneEntity(id);
      if (!res) {
          throw new HttpException(
            Constants.NOT_FOUND,
            HttpStatus.NOT_FOUND
          );
      }
      if (files && files.backgroundImage) {
          const backgroundImage: any = await this.doSpaceService.uploadFile(files.backgroundImage[0], "WeeklyAd/");
          dto.backgroundImage = backgroundImage;
      }

      const data: any = await this.repository.updateEntity(dto, id);
      if (!data) {
          throw new HttpException(
            Constants.UPDATE_FAILED,
            HttpStatus.BAD_REQUEST
          );
      }
        
      if (data) {
        await this.weeklyAdTagsListRepository.deleteWeeklyAdTagsList(data?.id);
        await this.weeklyAdTagsListRepository.bulkInsertWeeklyAdTagsList(JSON.parse(dto.tags));
      }
        
      return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async findAll(stateCode: string): Promise<WeeklyAdEntity[] | Error> {
      const resData: any = await this.repository.findByFilterQuery({ stateCode: stateCode });
      if (!resData) {
          throw new HttpException(
            Constants.NOT_FOUND,
            HttpStatus.NOT_FOUND
          );
      }
      

      let data = [];
      if (resData?.length > 0) {
        for (const da of resData) {
          const tags = await this.weeklyAdTagsListRepository.findByFilterQuery({ weeklyAdId: da.id });
          data.push({
            ...da,
            tags
          })
        }
      }
      
      return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<WeeklyAdEntity[] | Error> {
      const data = await this.repository.findOne({ id: id });
      if (!data) {
          throw new HttpException(
            Constants.NOT_FOUND,
            HttpStatus.NOT_FOUND
          );
      }
      if (data) {
        const tags = await this.weeklyAdTagsListRepository.findByFilterQuery({ weeklyAdId: data.id });

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", { ...data, tags });
      }
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}